#! /bin/sh

#
# Update modules script for United Linux 
#
# DESCRITION: Use this script to automatically update an existing
#             UnitedLinux installation with a new ServeRAID device driver.
#

GREP="/bin/grep"
SORT="/bin/sort"
CUT="/usr/bin/cut"
GUNZIP="/bin/gunzip"
MKINITRD="/sbin/mk_initrd"
LILO="/sbin/lilo"
CPIO="/bin/cpio"
MODULES="./update.tar.gz"
SUPPORTED_KERNELS=""

getSupportedKernels() {

        for i in `tar -tzvf $MODULES | $GREP "/modules/2" | cut -d ' ' -f10 | cut -d '/' -f3` ; do
                SUPPORTED_KERNELS="$i $SUPPORTED_KERNELS"
        done
}


updateKernel() {
        echo -n "Checking /etc/sysconfig/kernel file ... "

        # list of updated modules
        mods="ips"
        ipsPresent=`$GREP "INITRD_MODULES" /etc/sysconfig/kernel | $GREP $mods`

        if [ "$ipsPresent" ]; then
                echo "no update needed"
        else
                #
                # Backup kernel file
                #
                cp /etc/sysconfig/kernel /etc/sysconfig/kernel~
                # rebuild INITRD_MODULES info:
                #  - add all modules from $mods that are currently loaded
                imods=`$GREP "INITRD_MODULES=" /etc/sysconfig/kernel | sed -e 's/^INITRD_MODULES=//' -e 's/ *" *//g'`
                imods2=" $imods $mods "
                $GREP -v "INITRD_MODULES=" /etc/sysconfig/kernel > /etc/sysconfig/kernel.tmp
                mv /etc/sysconfig/kernel.tmp /etc/sysconfig/kernel
                echo "INITRD_MODULES="\""$imods2"\" >> /etc/sysconfig/kernel
                echo "completed successfully"
        fi
}


testToolsInstalled() {
	if [ ! -x $GUNZIP ]; then
		echo "This script needs the gzip package."
		echo "Please install the gzip package and rerun this script."
		exit 1
	fi

	if [ ! -x $CPIO ]; then
		echo "This script needs the cpio package."
		echo "Please install the cpio package and rerun this script."
		exit 1
	fi

	if [ ! -x $GREP ]; then
		echo "This script needs the grep package."
		echo "Please install the grep package and rerun this script."
		exit 1
	fi

	if [ ! -x $CUT -o ! -x $SORT ]; then
		echo "This script needs the textutils package."
		echo "Please install the textutils package and rerun this script."
		exit 1
	fi
}

installModules() {
	if [ ! -f $MODULES ]; then
		echo "Unable to locate update.tar.gz."
		echo -n "You must run this script from the SuSE device "
                echo "drivers for ServeRAID diskette."
		exit 1
	fi

	updated=0

	for i in $SUPPORTED_KERNELS; do
		echo -n "Updating kernel: $i ... "

		if [ -d /lib/modules/$i ]; then
  			#
			# Copy 2.2.x kernel module into module tree
			#
                        if [ -d /lib/modules/$i/scsi ]; then 
                                #
                                # Extract the module from the tarball
		     	        #
			        $GUNZIP -c $MODULES | (cd /tmp; $CPIO -id lib/modules/$i/scsi/ips.o > /dev/null 2>&1)
			        if [ ! -f /tmp/lib/modules/$i/scsi/ips.o ]; then
			           if [ ! -f /tmp/lib/modules/$i/kernel/drivers/scsi/ips.o ]; then
			                echo "error extracting module"
				        continue
	  		           fi
			        fi


			        cp /tmp/lib/modules/$i/scsi/ips.o /lib/modules/$i/scsi > /dev/null 2>&1
			        if [ $? -ne 0 ]; then
				        echo "error copying module"
				        continue
                                else
                                        echo "file copied"
                                        updated=1
			        fi
                        fi
                        
                        #
                        # With the 2.4.X kernel mk_initrd will look in the directory 
                        # /lib/modules/<kernel_version>/kernel/driver/scsi for the modules
                        #
                        if [ -d /lib/modules/$i/kernel/drivers/scsi ]; then  
                                #
                                # Extract the module from the tarball
		     	        #
			        $GUNZIP -c $MODULES | (cd /tmp; $CPIO -id lib/modules/$i/kernel/drivers/scsi/ips.o > /dev/null 2>&1)
			        if [ ! -f /tmp/lib/modules/$i/kernel/drivers/scsi/ips.o ]; then
			           if [ ! -f /tmp/lib/modules/$i/kernel/drivers/scsi/ips.o ]; then
			                echo "error extracting module"
				        continue
	  		           fi
			        fi

                                cp /tmp/lib/modules/$i/kernel/drivers/scsi/ips.o /lib/modules/$i/kernel/drivers/scsi > /dev/null 2>&1
			        if [ $? -ne 0 ]; then
				        echo "error copying module"
				        continue
                                else
                                        echo "file copied"
                                        updated=1
			        fi
                        fi

                        rm -fr  /tmp/lib/modules/$i > /dev/null 2>&1
			if [ $? -ne 0 ]; then
				echo "error removing /tmp/lib/modules/$i"
				continue
			fi

		else
			echo "not found"
		fi
	done

        #
        # Update /etc/sysconfig/kernel file
        #
        updateKernel

	#
	# Run mk_initrd to update initrd image
	#
	if [ $updated ]; then
                echo -n "Running mk_initrd ... "
	        $MKINITRD > /dev/null 2>&1
	        if [ $? -ne 0 ]; then
		        echo "error running mkinitrd"
		        exit 1
                else
			echo "completed successfully"
	        fi
        fi

        #
        # Run lilo - Don't show msgs; if LILO not installed, just let if fail and go on
        #
	if [ $updated ]; then
		$LILO > /dev/null 2>&1
	fi

	echo ""
	echo "Installation complete.  You must now reboot your server."
}

getSupportedKernels
testToolsInstalled
installModules

